@@ -25,7 +25,7 @@ import pytransit.__main__
 
 
 if __name__ == "__main__":
-    pytransit.__main__.main()
+    pytransit.__main__.run_main()
 
 
 
